package kaka.cakelight;

import kaka.cakelight.mode.Mode;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Console extends Thread {
    private boolean running;
    private CakeLight cakelight;
    private Configuration config;
    private Map<String, Command> commands = new HashMap<>();
    private List<Command> commandList = new ArrayList<>();

    public static Console start(CakeLight cakelight, Configuration config) {
	Console console = new Console(cakelight, config);
	console.start();
	return console;
    }

    private Console(CakeLight cakelight, Configuration config) {
        this.cakelight = cakelight;
	this.config = config;
	register(Commands.help());
	register(Commands.quit());
	register(Commands.push());
	register(Commands.pop());
	register(Commands.video());
	register(Commands.color());
	register(Commands.brightness());
	register(Commands.gamma());
	register(Commands.saturation());
	register(Commands.ambientMode());
	register(Commands.twoColorNoiseMode());
	register(Commands.sunriseMode());
    }

    public CakeLight getCakelight() {
	return cakelight;
    }

    public Configuration getConfig() {
	return config;
    }

    List<Command> getCommands() {
        return commandList;
    }

    void quit() {
        cakelight.turnOff();
        running = false;
    }

    private void register(Command cmd) {
	for (String name : cmd.getNames()){
	    if (commands.containsKey(name)) {
	        System.out.println("Command name '" + name + "' is already registered by '" + commands.get(name).getClass() + "'!");
	        System.exit(1);
	    }
	    commands.put(name, cmd);
	}
	commandList.add(cmd);
    }

    @Override
    public void run() {
	running = true;
	try (BufferedReader reader = new BufferedReader(new InputStreamReader(System.in))) {
	    while (running) {
		System.out.print("> ");
		String input = reader.readLine();
		handleInput(input);
	    }
	} catch (IOException e) {
	    System.out.println("Error reading from command line");
	}
    }

    void handleInput(String input) {
	Object obj = internalHandleInput(input);
	if (obj instanceof Mode) {
	    cakelight.setMode((Mode) obj);
	}
    }

    Object internalHandleInput(String input) {
	String[] splitInput = input.split("\\s+", 2);
	String name = splitInput[0];
	String[] args = splitInput.length == 2
		? splitInput[1].split("\\s+")
		: new String[]{};

	Command cmd = commands.get(name);
	if (cmd != null) {
	    return cmd.activate(this, args);
	} else {
	    out("no command named '" + name + "'");
	}
	return null;
    }

    void out(String text) {
	System.out.println("(" + text + ")");
    }

    Color parseColor(String s) {
	switch (s.toLowerCase()) {
	    case "r": return Color.rgb(255, 0, 0);
	    case "g": return Color.rgb(0, 255, 0);
	    case "b": return Color.rgb(0, 0, 255);
	    default: // assume hexadecimal
		if (s.startsWith("#")) {
		    s = s.substring(1);
		}
		if (s.length() == 3) {
		    return Color.rgb(
			    Integer.parseInt(s.substring(0, 1), 16) * 16 + Integer.parseInt(s.substring(0, 1), 16),
			    Integer.parseInt(s.substring(1, 2), 16) * 16 + Integer.parseInt(s.substring(1, 2), 16),
			    Integer.parseInt(s.substring(2, 3), 16) * 16 + Integer.parseInt(s.substring(2, 3), 16)
		    );
		} else if (s.length() == 6) {
		    return Color.rgb(
			    Integer.parseInt(s.substring(0, 2), 16),
			    Integer.parseInt(s.substring(2, 4), 16),
			    Integer.parseInt(s.substring(4, 6), 16)
		    );
		}
	}
	System.out.println("Failed to parse color '" + s + "'. Using black instead.");
	return Color.BLACK;
    }

    public interface Command {
        String[] getNames();
        Object activate(Console console, String[] args);
    }
}
